package edu.umd.dawn.common.annotations;

import edu.umd.dawn.common.exceptions.BaseExceptions;
import edu.umd.dawn.common.exceptions.DawnException;
import edu.umd.dawn.common.jwt.Claims;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Aspect
@Component
@Log4j2
@RequiredArgsConstructor(onConstructor = @__(@Autowired)) // Autowired annotated lombok generated constructor
public class RoleRestrictionAspect extends AspectBase<RoleRestriction> {

    private final HttpServletRequest request;

    @Value("${config.local}")
    private boolean local = false;

    @Around("@annotation(RoleRestriction)")
    public Object run(ProceedingJoinPoint jointPoint) throws Exception, Throwable {
        RoleRestriction annotation = grabAnnotation(jointPoint, RoleRestriction.class);
        if (!local) {
            Claims claims = (Claims) request.getAttribute("claims");
            if (claims == null) {
                throw new DawnException(BaseExceptions.FORBIDDEN);
            }
            // call db to check
        } else if (!annotation.disableWarning()) {
            log.warn("RoleRestriction annotation has been disabled - if this is a production environment, consider this"
                    + " a critical security error. Request with unknown role accessed endpoint with restriction "
                    + annotation.value().toString() + ".");
        }

        Object proceed = jointPoint.proceed();
        return proceed;
    }
}
