package edu.umd.dawn.common.exceptions;

import com.fasterxml.jackson.annotation.JsonIgnore;
import java.util.HashMap;
import java.util.Map;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import org.slf4j.MDC;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

@Getter
@Setter
@Builder
public class ExceptionResponse {

    private String source;
    private String errorCode;
    private String description;
    private Map<String, String> details;

    @JsonIgnore
    private DawnException exception;

    public ExceptionResponse buildDetails() {
        Map<String, String> details = new HashMap<>();
        details.put("requestId", MDC.get("requestId"));
        this.setDetails(details);
        return this;
    }

    public static ExceptionResponse fromDawnException(DawnException exception, String source) {
        return ExceptionResponse.builder()
                .source(source)
                .description(exception.getDescription())
                .errorCode(exception.getErrorCode())
                .exception(exception)
                .build()
                .buildDetails();
    }

    public ResponseEntity<Object> toResponse() {
        return new ResponseEntity<>(this, HttpStatus.valueOf(exception.getStatusCode()));
    }
}
